@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2022 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2025 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file in the Gmsh root directory for license information.
 // Please report all issues on https://gitlab.onelab.info/gmsh/gmsh/issues.
@@ -67,37 +67,35 @@ static void FinishUpBoundingBox()
   for(int i = 0; i < 3; i++)
     range[i] = CTX::instance()->max[i] - CTX::instance()->min[i];
 
-  if(range[0] < CTX::instance()->geom.tolerance &&
-     range[1] < CTX::instance()->geom.tolerance &&
-     range[2] < CTX::instance()->geom.tolerance) {
+  // OCC tolerance is 1e-7
+  double tol = std::max(1e-6, CTX::instance()->geom.tolerance);
+
+  if(range[0] < tol && range[1] < tol && range[2] < tol) {
     CTX::instance()->min[0] -= 1.;
     CTX::instance()->min[1] -= 1.;
     CTX::instance()->max[0] += 1.;
     CTX::instance()->max[1] += 1.;
   }
-  else if(range[0] < CTX::instance()->geom.tolerance &&
-          range[1] < CTX::instance()->geom.tolerance) {
+  else if(range[0] < tol && range[1] < tol) {
     CTX::instance()->min[0] -= range[2];
     CTX::instance()->min[1] -= range[2];
     CTX::instance()->max[0] += range[2];
     CTX::instance()->max[1] += range[2];
   }
-  else if(range[0] < CTX::instance()->geom.tolerance &&
-          range[2] < CTX::instance()->geom.tolerance) {
+  else if(range[0] < tol && range[2] < tol) {
     CTX::instance()->min[0] -= range[1];
     CTX::instance()->max[0] += range[1];
   }
-  else if(range[1] < CTX::instance()->geom.tolerance &&
-          range[2] < CTX::instance()->geom.tolerance) {
+  else if(range[1] < tol && range[2] < tol) {
     CTX::instance()->min[1] -= range[0];
     CTX::instance()->max[1] += range[0];
   }
-  else if(range[0] < CTX::instance()->geom.tolerance) {
+  else if(range[0] < tol) {
     double l = sqrt(SQU(range[1]) + SQU(range[2]));
     CTX::instance()->min[0] -= l;
     CTX::instance()->max[0] += l;
   }
-  else if(range[1] < CTX::instance()->geom.tolerance) {
+  else if(range[1] < tol) {
     double l = sqrt(SQU(range[0]) + SQU(range[2]));
     CTX::instance()->min[1] -= l;
     CTX::instance()->max[1] += l;
@@ -185,9 +183,13 @@ int ParseFile(const std::string &fileName, bool close, bool errorIfMissing)
   // fsetpos/fgetpos (used e.g. for user-defined functions)
   FILE *fp;
   if(!(fp = Fopen(fileName.c_str(), "rb"))) {
-    if(errorIfMissing)
+    if(errorIfMissing) {
       Msg::Error("Unable to open file '%s'", fileName.c_str());
-    return 0;
+      return 0;
+    }
+    else{
+      return 1; // still ok
+    }
   }
 
   Msg::AddOnelabStringChoice("Gmsh/}Input files", "file", fileName, true, true,
@@ -216,8 +218,8 @@ int ParseFile(const std::string &fileName, bool close, bool errorIfMissing)
   }
 
   if(close) {
-    gmsh_yyflush();
     fclose(gmsh_yyin);
+    gmsh_yyflush();
   }
   else {
     openedFiles.push_back(gmsh_yyin);
@@ -239,7 +241,7 @@ int ParseFile(const std::string &fileName, bool close, bool errorIfMissing)
 #endif
 }
 
-static bool doSystemUncompress(const std::string &fileName,
+static bool DoSystemUncompress(const std::string &fileName,
                                const std::string &noExt)
 {
   std::ostringstream sstream;
@@ -275,7 +277,7 @@ void ParseString(const std::string &str, bool inCurrentModelDir)
 }
 
 #if defined(HAVE_ONELAB)
-static std::string getSolverForExtension(const std::string &ext)
+static std::string GetSolverForExtension(const std::string &ext)
 {
   for(int i = 0; i < NUM_SOLVERS; i++) {
     if(opt_solver_extension(i, GMSH_GET, "") == ext)
@@ -284,7 +286,7 @@ static std::string getSolverForExtension(const std::string &ext)
   return "";
 }
 
-static int defineSolver(const std::string &name)
+static int DefineSolver(const std::string &name)
 {
   int i;
   for(i = 0; i < NUM_SOLVERS; i++) {
@@ -296,16 +298,19 @@ static int defineSolver(const std::string &name)
 #endif
 
 int MergeFile(const std::string &fileName, bool errorIfMissing,
-              bool setBoundingBox, bool importPhysicalsInOnelab,
-              int partitionToRead)
+              bool setBoundingBox, bool importPhysicalsInOnelab)
 {
   // added 'b' for pure Windows programs, since some of these files
   // contain binary data
   FILE *fp = Fopen(fileName.c_str(), "rb");
   if(!fp) {
-    if(errorIfMissing)
+    if(errorIfMissing) {
       Msg::Error("Unable to open file '%s'", fileName.c_str());
-    return 0;
+      return 0;
+    }
+    else{
+      return 1; // still ok
+    }
   }
 
   char header[256];
@@ -324,7 +329,7 @@ int MergeFile(const std::string &fileName, bool errorIfMissing,
   std::string noExt = split[0] + split[1], ext = split[2];
 
   if(ext == ".gz") {
-    if(doSystemUncompress(fileName, noExt)) return MergeFile(noExt, false);
+    if(DoSystemUncompress(fileName, noExt)) return MergeFile(noExt, false);
   }
 
   CTX::instance()->geom.draw = 0; // don't try to draw the model while reading
@@ -335,9 +340,9 @@ int MergeFile(const std::string &fileName, bool errorIfMissing,
   int status = 0;
 
 #if defined(HAVE_ONELAB)
-  std::string solver = getSolverForExtension(ext);
+  std::string solver = GetSolverForExtension(ext);
   if(solver.size()) {
-    int num = defineSolver(solver);
+    int num = DefineSolver(solver);
     Msg::SetOnelabString(solver + "/Model name", fileName, true, true, false, 3,
                          "file");
     if(GModel::current()->getName() == "" ||
@@ -352,7 +357,7 @@ int MergeFile(const std::string &fileName, bool errorIfMissing,
   }
   else if(ext == ".py" || ext == ".PY" || ext == ".m" || ext == ".M" ||
           ext == ".exe" || ext == ".EXE") {
-    int num = defineSolver(split[1]);
+    int num = DefineSolver(split[1]);
     opt_solver_executable(num, GMSH_SET, fileName);
     CTX::instance()->launchSolverAtStartup = num;
     CTX::instance()->geom.draw = 1;
@@ -369,9 +374,16 @@ int MergeFile(const std::string &fileName, bool errorIfMissing,
     status =
       GModel::current()->readSTL(fileName, CTX::instance()->geom.tolerance);
   }
+  else if(ext == ".NII" || ext == ".nii") {
+    status =
+      GModel::current()->readNII(fileName);
+  }
   else if(ext == ".brep" || ext == ".rle" || ext == ".brp" || ext == ".BRP") {
     status = GModel::current()->readOCCBREP(fileName);
   }
+  else if(ext == ".xao" || ext == ".XAO") {
+    status = GModel::current()->readOCCXAO(fileName);
+  }
   else if(ext == ".iges" || ext == ".IGES" || ext == ".igs" || ext == ".IGS") {
     status = GModel::current()->readOCCIGES(fileName);
   }
@@ -403,6 +415,9 @@ int MergeFile(const std::string &fileName, bool errorIfMissing,
   else if(ext == ".off" || ext == ".OFF") {
     status = GModel::current()->readOFF(fileName);
   }
+  else if(ext == ".obj" || ext == ".OBJ") {
+    status = GModel::current()->readOBJ(fileName);
+  }
   else if(ext == ".diff" || ext == ".DIFF") {
     status = GModel::current()->readDIFF(fileName);
   }
@@ -419,6 +434,9 @@ int MergeFile(const std::string &fileName, bool errorIfMissing,
   else if(ext == ".bdf" || ext == ".BDF" || ext == ".nas" || ext == ".NAS") {
     status = GModel::current()->readBDF(fileName);
   }
+  else if (ext == ".neu" || ext == ".NEU") {
+    status=GModel::current()->readNEU(fileName);
+  }
   else if(ext == ".dat" || ext == ".DAT") {
     if(!strncmp(header, "BEGIN ACTRAN", 12))
       status = GModel::current()->readACTRAN(fileName);
@@ -511,9 +529,6 @@ int MergeFile(const std::string &fileName, bool errorIfMissing,
         CTX::instance()->geom.matchMeshScaleFactor = 1;
         status = GModel::current()->readMSH(fileName);
       }
-#if defined(HAVE_POST)
-      if(status > 1) status = PView::readMSH(fileName, -1, partitionToRead);
-#endif
     }
 #if defined(HAVE_POST)
     else if(ext == ".pch") {
@@ -781,6 +796,10 @@ void OpenProject(const std::string &fileName, bool errorIfMissing)
 
   CTX::instance()->lock = 0;
 
+  // FIXME: temporary for auto-extrude testing
+  if(CTX::instance()->geom.autoExtrude)
+    GModel::current()->addAutomaticExtrusionConstraints({10}, {1}, true, {});
+
 #if defined(HAVE_FLTK)
   if(FlGui::available()) {
     file_watch_cb(nullptr, nullptr);
