@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2022 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2025 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file in the Gmsh root directory for license information.
 // Please report all issues on https://gitlab.onelab.info/gmsh/gmsh/issues.
@@ -99,7 +99,7 @@ public:
   // check if the GUI is available
   static bool available();
   // run the GUI until there's no window left
-  static int run();
+  static int run(const std::string &optionFileName="");
   // check if there are any pending events, and process them (if rateLimited is
   // set, only perform the check if one has not been made in the last 1 /
   // General.FltkRefreshRate seconds)
@@ -134,6 +134,8 @@ public:
   void updateFields();
   // reset the visibility window
   void resetVisibility();
+  // update the statistics window
+  void updateStatistics(bool qualities = false);
   // store current window positions and sizes in CTX
   void storeCurrentWindowsInfo();
   // get the last opengl window that received an event
@@ -161,6 +163,8 @@ public:
   void saveMessages(const char *fileName);
   // rebuild the tree
   void rebuildTree(bool deleteWidgets);
+  // toggles the module open or closed based on its current state
+  void toggleModule(const std::string &name);
   // open module in tree
   void openModule(const std::string &name);
   // open tree item
