@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2022 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2025 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file in the Gmsh root directory for license information.
 // Please report all issues on https://gitlab.onelab.info/gmsh/gmsh/issues.
@@ -22,10 +22,6 @@
 #include "MFaceHash.h"
 #include "MEdgeHash.h"
 
-#define hashmapMFace                                                           \
-  std::unordered_map<MFace, std::size_t, MFaceHash, MFaceEqual>
-#define hashmapMEdge                                                           \
-  std::unordered_map<MEdge, std::size_t, MEdgeHash, MEdgeEqual>
 
 template <class scalar> class simpleFunction;
 
@@ -42,7 +38,11 @@ class MElementOctree;
 
 // A geometric model. The model is a "not yet" non-manifold B-Rep.
 class GModel {
+public:
+  using hashmapMFace = std::unordered_map<MFace, std::size_t, MFaceHash, MFaceEqual>;
+  using hashmapMEdge = std::unordered_map<MEdge, std::size_t, MEdgeHash, MEdgeEqual>;
 private:
+
   std::multimap<std::pair<const std::vector<int>, const std::vector<int> >,
                 std::pair<const std::string, const std::vector<int> > >
     _homologyRequests;
@@ -99,9 +99,9 @@ protected:
   // vertex and element caches to speed-up direct access by tag (mostly
   // used for post-processing I/O)
   std::vector<MVertex *> _vertexVectorCache;
-  std::map<int, MVertex *> _vertexMapCache;
+  std::map<std::size_t, MVertex *> _vertexMapCache;
   std::vector<std::pair<MElement *, int> > _elementVectorCache;
-  std::map<int, std::pair<MElement *, int> > _elementMapCache;
+  std::map<std::size_t, std::pair<MElement *, int> > _elementMapCache;
   std::map<int, int> _elementIndexCache;
 
   // ghost cell information (stores partitions for each element acting
@@ -170,7 +170,7 @@ protected:
 
   // store the vertices in the geometrical entity they are associated
   // with, and delete those that are not associated with any entity
-  void _storeVerticesInEntities(std::map<int, MVertex *> &vertices);
+  void _storeVerticesInEntities(std::map<std::size_t, MVertex *> &vertices);
   void _storeVerticesInEntities(std::vector<MVertex *> &vertices);
 
   // store the physical tags in the geometrical entities
@@ -274,8 +274,13 @@ public:
 
   // add a mesh edge or face in the global edge or face map with number "num",
   // or number it (starting at 1) if num == 0
-  std::size_t addMEdge(MEdge &edge, std::size_t num = 0);
-  std::size_t addMFace(MFace &face, std::size_t num = 0);
+  std::size_t addMEdge(MEdge &&edge, std::size_t num = 0);
+  std::size_t addMFace(MFace &&face, std::size_t num = 0);
+
+  // get number of edges or faces
+  std::size_t getNumMEdges() const { return _mapEdgeNum.size(); };
+  std::size_t getNumMFaces() const { return _mapFaceNum.size(); };
+
   // get the edge of face and its global number given mesh nodes (return 0 if
   // the edge or face does not exist in the edge or face map)
   std::size_t getMEdge(MVertex *v0, MVertex *v1, MEdge &edge);
@@ -289,8 +294,10 @@ public:
 
   // renumber mesh vertices and elements in a continuous sequence (this
   // invalidates the mesh caches)
-  void renumberMeshVertices();
-  void renumberMeshElements();
+  void renumberMeshVertices(const std::map<std::size_t, std::size_t> &mapping =
+                            std::map<std::size_t, std::size_t>());
+  void renumberMeshElements(const std::map<std::size_t, std::size_t> &mapping =
+                            std::map<std::size_t, std::size_t>());
 
   // delete all the mesh-related caches (this must be called when the
   // mesh is changed)
@@ -300,6 +307,8 @@ public:
   void deleteMesh(const std::vector<GEntity *> &entities);
   // delete the vertex arrays used for efficient mesh drawing
   void deleteVertexArrays();
+  // delete the vertex arrays used for efficient geometry drawing
+  void deleteGeometryVertexArrays();
 
   // remove all mesh vertex associations to geometrical entities and remove
   // vertices from geometrical entities, then _associateEntityWithMeshVertices
@@ -434,6 +443,8 @@ public:
   getPhysicalGroups(std::map<int, std::vector<GEntity *> > groups[4]) const;
   void getPhysicalGroups(int dim,
                          std::map<int, std::vector<GEntity *> > &groups) const;
+  void getEntitiesForPhysicalName(const std::string &name,
+                                  std::vector<GEntity *> &entities) const;
   const std::map<std::pair<int, int>, std::string> &getPhysicalNames() const
   {
     return _physicalNames;
@@ -535,12 +546,12 @@ public:
                                                  bool strict = true);
 
   // access a mesh element by tag, using the element cache
-  MElement *getMeshElementByTag(int n)
+  MElement *getMeshElementByTag(std::size_t n)
   {
-    int tag;
-    return getMeshElementByTag(n, tag);
+    int entityTag;
+    return getMeshElementByTag(n, entityTag);
   }
-  MElement *getMeshElementByTag(int n, int &entityTag);
+  MElement *getMeshElementByTag(std::size_t n, int &entityTag);
 
   // access temporary mesh element index
   int getMeshElementIndex(MElement *e);
@@ -558,7 +569,7 @@ public:
   void rebuildMeshElementCache(bool onlyIfNecessary = false);
 
   // access a mesh vertex by tag, using the vertex cache
-  MVertex *getMeshVertexByTag(int n);
+  MVertex *getMeshVertexByTag(std::size_t n);
 
   // add a mesh vertex to the global mesh vertex cache
   void addMVertexToVertexCache(MVertex* v);
@@ -749,10 +760,13 @@ public:
 
   // OCC model
   int readOCCBREP(const std::string &name);
+  int readOCCXAO(const std::string &name);
   int readOCCSTEP(const std::string &name);
   int readOCCIGES(const std::string &name);
-  int writeOCCSTEP(const std::string &name);
   int writeOCCBREP(const std::string &name);
+  int writeOCCXAO(const std::string &name);
+  int writeOCCSTEP(const std::string &name);
+  int writeOCCIGES(const std::string &name);
   int importOCCShape(const void *shape);
   GVertex *getVertexForOCCShape(const void *shape);
   GEdge *getEdgeForOCCShape(const void *shape);
@@ -796,6 +810,12 @@ public:
                bool saveAll = false, double scalingFactor = 1.0,
                int oneSolidPerSurface = 0);
 
+
+  // NII format is created by Neuroimaging Informatics Technology Initiative. It is commonly used to store magnetic resonance imaging (MRI) data.
+  int readNII(const std::string &name, float isolevel = 0.0, int isoDarkMediumBright123 = 2,
+	      float reduceFraction = 0.25, int preSmooth = 1, bool onlyLargest = true,
+	      bool fillBubbles = false, int postSmooth = 1);
+
   // X3D (only output from OCCT's triangulation)
   int writeX3D(const std::string &name, bool saveAll = false,
                double scalingFactor = 1.0, int x3dsurfaces = 1,
@@ -828,6 +848,11 @@ public:
   int writeOFF(const std::string &name, bool saveAll = false,
                double scalingFactor = 1.0);
 
+  // Wavefront OBJ format
+  int readOBJ(const std::string &name);
+  int writeOBJ(const std::string &name, bool saveAll = false,
+               double scalingFactor = 1.0);
+
   // Nastran Bulk Data File format
   int readBDF(const std::string &name);
   int writeBDF(const std::string &name, int format = 0, int elementTagType = 1,
@@ -905,6 +930,7 @@ public:
   int writeSU2(const std::string &name, bool saveAll, double scalingFactor);
 
   // GAMBIT neutral mesh file (.neu)
+  int readNEU(const std::string &name);
   int writeNEU(const std::string &name, bool saveAll, double scalingFactor);
 };
 
