@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2022 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2025 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file in the Gmsh root directory for license information.
 // Please report all issues on https://gitlab.onelab.info/gmsh/gmsh/issues.
@@ -356,8 +356,8 @@ void MVertex::writeBDF(FILE *fp, int format, double scalingFactor)
   }
   else {
     // large field format (8 char first/last field, 16 char middle, 6 per line)
-    fprintf(fp, "GRID*   %-16ld%-16d%-16.9G%-16.9G\n", _index, 0, x1, y1);
-    fprintf(fp, "*       %-16.9G\n", z1);
+    fprintf(fp, "GRID*   %-16ld%-16d%-#16.9G%-#16.9G\n", _index, 0, x1, y1);
+    fprintf(fp, "*       %-#16.9G\n", z1);
   }
 }
 
@@ -377,6 +377,14 @@ void MVertex::writeKEY(FILE *fp, double scalingFactor)
           y() * scalingFactor, z() * scalingFactor);
 }
 
+void MVertex::writeRAD(FILE *fp, double scalingFactor)
+{
+  if(_index < 0) return; // negative index vertices are never saved
+
+  fprintf(fp, "%10ld%20g%20g%20g\n", _index, x() * scalingFactor,
+          y() * scalingFactor, z() * scalingFactor);
+}
+
 void MVertex::writeDIFF(FILE *fp, bool binary, double scalingFactor)
 {
   if(_index < 0) return; // negative index vertices are never saved
