@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2022 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2025 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file in the Gmsh root directory for license information.
 // Please report all issues on https://gitlab.onelab.info/gmsh/gmsh/issues.
@@ -31,6 +31,7 @@
 #include "surfaceFiller.h"
 #endif
 
+static constexpr double ONE_THIRD = 1.0 / 3.0;
 static double LIMIT_ = 0.5 * std::sqrt(2.0) * 1;
 int MTri3::radiusNorm = 2;
 
@@ -133,7 +134,7 @@ void _printTris(char *name, ITERATOR it, ITERATOR end, bidimMeshData *data,
               degenerated->end();
             if(deg[0] && !deg[1] && !deg[2]) {
               fprintf(
-                ff, "SQ(%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g) {%d,%d,%d,%d};\n",
+                ff, "SQ(%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g) {%lu,%lu,%lu,%lu};\n",
                 u3, v1, 0., u2, v1, 0., u2, v2, 0., u3, v3, 0.,
                 (worst)->tri()->getVertex(0)->getNum(),
                 (worst)->tri()->getVertex(0)->getNum(),
@@ -142,7 +143,7 @@ void _printTris(char *name, ITERATOR it, ITERATOR end, bidimMeshData *data,
             }
             else if(!deg[0] && deg[1] && !deg[2]) {
               fprintf(
-                ff, "SQ(%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g) {%d,%d,%d,%d};\n",
+                ff, "SQ(%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g) {%lu,%lu,%lu,%lu};\n",
                 u1, v2, 0., u3, v2, 0., u3, v3, 0., u1, v1, 0.,
                 (worst)->tri()->getVertex(1)->getNum(),
                 (worst)->tri()->getVertex(1)->getNum(),
@@ -151,7 +152,7 @@ void _printTris(char *name, ITERATOR it, ITERATOR end, bidimMeshData *data,
             }
             else if(!deg[0] && !deg[1] && deg[2]) {
               fprintf(
-                ff, "SQ(%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g) {%d,%d,%d,%d};\n",
+                ff, "SQ(%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g) {%lu,%lu,%lu,%lu};\n",
                 u2, v3, 0., u1, v3, 0., u1, v1, 0., u2, v2, 0.,
                 (worst)->tri()->getVertex(2)->getNum(),
                 (worst)->tri()->getVertex(2)->getNum(),
@@ -159,7 +160,7 @@ void _printTris(char *name, ITERATOR it, ITERATOR end, bidimMeshData *data,
                 (worst)->tri()->getVertex(1)->getNum());
             }
             else if(!deg[0] && !deg[1] && !deg[2]) {
-              fprintf(ff, "ST(%g,%g,%g,%g,%g,%g,%g,%g,%g) {%d,%d,%d};\n", u1,
+              fprintf(ff, "ST(%g,%g,%g,%g,%g,%g,%g,%g,%g) {%lu,%lu,%lu};\n", u1,
                       v1, 0., u2, v2, 0., u3, v3, 0.,
                       (worst)->tri()->getVertex(0)->getNum(),
                       (worst)->tri()->getVertex(1)->getNum(),
@@ -167,7 +168,7 @@ void _printTris(char *name, ITERATOR it, ITERATOR end, bidimMeshData *data,
             }
           }
           else {
-            fprintf(ff, "ST(%g,%g,%g,%g,%g,%g,%g,%g,%g) {%d,%d,%d};\n", u1, v1,
+            fprintf(ff, "ST(%g,%g,%g,%g,%g,%g,%g,%g,%g) {%lu,%lu,%lu};\n", u1, v1,
                     0., u2, v2, 0., u3, v3, 0.,
                     (worst)->tri()->getVertex(0)->getNum(),
                     (worst)->tri()->getVertex(1)->getNum(),
@@ -175,7 +176,7 @@ void _printTris(char *name, ITERATOR it, ITERATOR end, bidimMeshData *data,
           }
         }
         else
-          fprintf(ff, "ST(%g,%g,%g,%g,%g,%g,%g,%g,%g) {%d,%d,%d};\n",
+          fprintf(ff, "ST(%g,%g,%g,%g,%g,%g,%g,%g,%g) {%lu,%lu,%lu};\n",
                   (worst)->tri()->getVertex(0)->x(),
                   (worst)->tri()->getVertex(0)->y(),
                   (worst)->tri()->getVertex(0)->z(),
@@ -331,11 +332,11 @@ static void circumCenterMetric(MTriangle *base, const double *metric,
 
 void buildMetric(GFace *gf, double *uv, double *metric)
 {
-  Pair<SVector3, SVector3> der = gf->firstDer(SPoint2(uv[0], uv[1]));
+  std::pair<SVector3, SVector3> der = gf->firstDer(SPoint2(uv[0], uv[1]));
 
-  metric[0] = dot(der.first(), der.first());
-  metric[1] = dot(der.second(), der.first());
-  metric[2] = dot(der.second(), der.second());
+  metric[0] = dot(der.first, der.first);
+  metric[1] = dot(der.second, der.first);
+  metric[2] = dot(der.second, der.second);
 }
 
 static double computeTolerance(const double radius)
@@ -370,8 +371,8 @@ int inCircumCircleAniso(GFace *gf, MTriangle *base, const double *uv,
     int index0 = data.getIndex(base->getVertex(0));
     int index1 = data.getIndex(base->getVertex(1));
     int index2 = data.getIndex(base->getVertex(2));
-    double pa[2] = {(data.Us[index0] + data.Us[index1] + data.Us[index2]) / 3.,
-                    (data.Vs[index0] + data.Vs[index1] + data.Vs[index2]) / 3.};
+    double pa[2] = {(data.Us[index0] + data.Us[index1] + data.Us[index2]) * ONE_THIRD,
+                    (data.Vs[index0] + data.Vs[index1] + data.Vs[index2]) * ONE_THIRD};
     buildMetric(gf, pa, metric);
   }
   else {
@@ -391,6 +392,23 @@ int inCircumCircleAniso(GFace *gf, MTriangle *base, const double *uv,
   return d3 < Radius2;
 }
 
+static void fourthPoint(double *p1, double *p2, double *p3, double *p4)
+{
+  double c[3];
+  circumCenterXYZ(p1, p2, p3, c);
+  double vx[3] = {p2[0] - p1[0], p2[1] - p1[1], p2[2] - p1[2]};
+  double vy[3] = {p3[0] - p1[0], p3[1] - p1[1], p3[2] - p1[2]};
+  double vz[3];
+  prodve(vx, vy, vz);
+  norme(vz);
+  double R =
+    sqrt((p1[0] - c[0]) * (p1[0] - c[0]) + (p1[1] - c[1]) * (p1[1] - c[1]) +
+         (p1[2] - c[2]) * (p1[2] - c[2]));
+  p4[0] = c[0] + R * vz[0];
+  p4[1] = c[1] + R * vz[1];
+  p4[2] = c[2] + R * vz[2];
+}
+
 MTri3::MTri3(MTriangle *t, double lc, SMetric3 *metric, bidimMeshData *data,
              GFace *gf)
   : deleted(false), base(t)
@@ -421,8 +439,8 @@ MTri3::MTri3(MTriangle *t, double lc, SMetric3 *metric, bidimMeshData *data,
       double const p2[2] = {data->Us[index1], data->Vs[index1]};
       double const p3[2] = {data->Us[index2], data->Vs[index2]};
 
-      double midpoint[2] = {(p1[0] + p2[0] + p3[0]) / 3.0,
-                            (p1[1] + p2[1] + p3[1]) / 3.0};
+      double midpoint[2] = {(p1[0] + p2[0] + p3[0]) * ONE_THIRD,
+                            (p1[1] + p2[1] + p3[1]) * ONE_THIRD};
 
       double quadAngle =
         backgroundMesh::current() ?
@@ -712,7 +730,6 @@ static int insertVertexB(std::list<edgeXface> &shell,
     int index0 = data.getIndex(t->getVertex(0));
     int index1 = data.getIndex(t->getVertex(1));
     int index2 = data.getIndex(t->getVertex(2));
-    constexpr double ONE_THIRD = 1. / 3.;
     double lc = ONE_THIRD * (data.vSizes[index0] + data.vSizes[index1] +
                              data.vSizes[index2]);
     double lcBGM =
@@ -868,8 +885,8 @@ static MTri3 *search4Triangle(MTri3 *t, double pt[2], bidimMeshData &data,
     int index0 = data.getIndex(t->tri()->getVertex(0));
     int index1 = data.getIndex(t->tri()->getVertex(1));
     int index2 = data.getIndex(t->tri()->getVertex(2));
-    SPoint3 q2((data.Us[index0] + data.Us[index1] + data.Us[index2]) / 3.0,
-               (data.Vs[index0] + data.Vs[index1] + data.Vs[index2]) / 3.0, 0);
+    SPoint3 q2((data.Us[index0] + data.Us[index1] + data.Us[index2]) * ONE_THIRD,
+               (data.Vs[index0] + data.Vs[index1] + data.Vs[index2]) * ONE_THIRD, 0);
     int i;
     for(i = 0; i < 3; i++) {
       int i1 = data.getIndex(t->tri()->getVertex(i == 0 ? 2 : i - 1));
@@ -879,7 +896,7 @@ static MTri3 *search4Triangle(MTri3 *t, double pt[2], bidimMeshData &data,
       if(intersection_segments_2(p1, p2, q1, q2)) break;
     }
     if(i >= 3) {
-      printf("impossible\n");
+      Msg::Warning("Impossible case in triangle search");
       break;
     }
     t = t->getNeigh(i);
@@ -1031,14 +1048,14 @@ void bowyerWatson(GFace *gf, int MAXPNT,
   }
 
   int ITER = 0;
-  int NBDELETED = 0;
+  //int NBDELETED = 0;
   while(1) {
     MTri3 *worst = *AllTris.begin();
     if(worst->isDeleted()) {
       delete worst->tri();
       delete worst;
       AllTris.erase(AllTris.begin());
-      NBDELETED++;
+      //NBDELETED++;
     }
     else {
       if(ITER++ % 5000 == 0) {
@@ -1058,8 +1075,8 @@ void bowyerWatson(GFace *gf, int MAXPNT,
       int index1 = DATA.getIndex(base->getVertex(1));
       int index2 = DATA.getIndex(base->getVertex(2));
       double pa[2] = {
-        (DATA.Us[index0] + DATA.Us[index1] + DATA.Us[index2]) / 3.,
-        (DATA.Vs[index0] + DATA.Vs[index1] + DATA.Vs[index2]) / 3.};
+        (DATA.Us[index0] + DATA.Us[index1] + DATA.Us[index2]) * ONE_THIRD,
+        (DATA.Vs[index0] + DATA.Vs[index1] + DATA.Vs[index2]) * ONE_THIRD};
 
       buildMetric(gf, pa, metric);
       circumCenterMetric(worst->tri(), metric, DATA, center, r2);
@@ -1153,8 +1170,8 @@ static double optimalPointFrontal(GFace *gf, MTri3 *worst, int active_edge,
   int index0 = data.getIndex(base->getVertex(0));
   int index1 = data.getIndex(base->getVertex(1));
   int index2 = data.getIndex(base->getVertex(2));
-  double pa[2] = {(data.Us[index0] + data.Us[index1] + data.Us[index2]) / 3.,
-                  (data.Vs[index0] + data.Vs[index1] + data.Vs[index2]) / 3.};
+  double pa[2] = {(data.Us[index0] + data.Us[index1] + data.Us[index2]) * ONE_THIRD,
+                  (data.Vs[index0] + data.Vs[index1] + data.Vs[index2]) * ONE_THIRD};
   buildMetric(gf, pa, metric);
   circumCenterMetric(worst->tri(), metric, data, center, r2);
   // compute the middle point of the edge
@@ -1235,6 +1252,7 @@ static bool optimalPointFrontalB(GFace *gf, MTri3 *worst, int active_edge,
                v3->z() - middle.z());
   SVector3 n1 = crossprod(v1v2, tmp);
   if(n1.norm() < 1.e-12) return true;
+
   SVector3 n2 = crossprod(n1, v1v2);
   n1.normalize();
   n2.normalize();
@@ -1298,13 +1316,16 @@ void bowyerWatsonFrontal(GFace *gf, std::map<MVertex *, MVertex *> *equivalence,
 
   Range<double> RU = gf->parBounds(0);
   Range<double> RV = gf->parBounds(1);
-  SPoint2 FAR(2 * RU.high(), 2 * RV.high());
+  // THIS WAS ACTUALLY WRONG IF high is 0 !!!
+  //  SPoint2 FAR(2 * RU.high(), 2 * RV.high());
+  /// This is better !
+  SPoint2 FAR(RU.high() + (RU.high()-RU.low()), RV.high() + (RV.high()-RV.low()));
 
 
   // insert points
-  int ITERATION = 0;
+  //int ITERATION = 0;
   while(1) {
-    ++ITERATION;
+    //++ITERATION;
     // if(ITERATION % 1 == 0 && Msg::GetVerbosity() == 99){
     //   char name[245];
     //   sprintf(name,"delFrontal_GFace_%d_Layer_%d.pos",gf->tag(),ITERATION);
@@ -1334,6 +1355,9 @@ void bowyerWatsonFrontal(GFace *gf, std::map<MVertex *, MVertex *> *equivalence,
           insertAPoint(gf, AllTris.end(), newPoint, metric, DATA, AllTris,
                        &ActiveTris, worst, nullptr, testStarShapeness);
       }
+      else {
+        Msg::Debug("no point found");
+      }
     }
   }
 
@@ -1401,7 +1425,8 @@ static void optimalPointFrontalQuad(GFace *gf, MTri3 *worst, int active_edge,
   const double rhoM_hat =
     std::min(std::max(rhoM, p), (p * p + q * q) / (2 * q));
   const double factor =
-    (rhoM_hat + std::sqrt(rhoM_hat * rhoM_hat - p * p)) / (std::sqrt(3.) * p);
+    (rhoM_hat + std::sqrt(std::max(rhoM_hat * rhoM_hat - p * p, 0.))) /
+    (std::sqrt(3.) * p);
 
   double npx, npy;
   if(xp * yp > 0) {
@@ -1618,6 +1643,7 @@ void bowyerWatsonParallelograms(
     return;
   }
 
+
 #if defined(HAVE_DOMHEX)
   if(old_algo_hexa()) {
     Msg::Debug("bowyerWatsonParallelograms: call packingOfParallelograms()");
@@ -1632,7 +1658,7 @@ void bowyerWatsonParallelograms(
   Msg::Error("Packing of parallelograms algorithm requires DOMHEX");
 #endif
 
-  Msg::Info("%lu Nodes created --> now staring insertion", packed.size());
+  Msg::Info("%zu Nodes created --> now staring insertion", packed.size());
 
   if(!buildMeshGenerationDataStructures(gf, AllTris, DATA)) {
     Msg::Error("Invalid meshing data structure");
@@ -1681,6 +1707,12 @@ void bowyerWatsonParallelograms(
     }
   }
 
+#if 0
+   char name[256];
+   sprintf(name,"RawTriangulation%d.pos",gf->tag());
+   _printTris (name, AllTris.begin(), AllTris.end(),nullptr);
+#endif
+
   transferDataStructure(gf, AllTris, DATA);
   backgroundMesh::unset();
 
@@ -1826,9 +1858,6 @@ void delaunayMeshIn2D(std::vector<MVertex *> &v,
   MVertex *box[4];
   initialSquare(v, box, t);
 
-  double AVG_ITER = 0;
-  double AVG_CAVSIZE = 0;
-
   if(hilbertSort) SortHilbert(v);
 
   for(size_t i = 0; i < v.size(); i++) {
@@ -1836,7 +1865,6 @@ void delaunayMeshIn2D(std::vector<MVertex *> &v,
 
     int NITER = 0;
     MTri3 *found = getTriToBreak(pv, t, NITER);
-    AVG_ITER += (double)NITER;
     if(!found) {
       Msg::Error("Cannot insert a point in 2D Delaunay");
       continue;
@@ -1845,7 +1873,6 @@ void delaunayMeshIn2D(std::vector<MVertex *> &v,
     cavity.clear();
 
     recurFindCavity(shell, cavity, pv, found);
-    AVG_CAVSIZE += (double)cavity.size();
 
     std::vector<MTri3 *> extended_cavity;
     for(std::size_t count = 0; count < shell.size(); count++) {
