@@ -1,10 +1,11 @@
-// Gmsh - Copyright (C) 1997-2022 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2025 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file in the Gmsh root directory for license information.
 // Please report all issues on https://gitlab.onelab.info/gmsh/gmsh/issues.
 
 #include <map>
 #include <set>
+#include <stdexcept>
 
 #include "GmshConfig.h"
 #include "meshGRegionHxt.h"
@@ -23,10 +24,20 @@
 
 #if defined(HAVE_HXT)
 
-extern "C" {
 #include "hxt_omp.h"
+extern "C" {
 #include "hxt_tetMesh.h"
 #include "hxt_tetDelaunay.h"
+#include "hxt_boundary_recovery.h"
+}
+
+static int getNumThreads()
+{
+  int nthreads = CTX::instance()->numThreads;
+  if(CTX::instance()->mesh.maxNumThreads3D > 0)
+    nthreads = CTX::instance()->mesh.maxNumThreads3D;
+  if(!nthreads) nthreads = Msg::GetMaxThreads();
+  return nthreads;
 }
 
 static HXTStatus messageCallback(HXTMessage *msg)
@@ -43,24 +54,35 @@ static HXTStatus nodalSizesCallBack(double *pts, uint32_t *volume,
   double lcGlob = CTX::instance()->lc;
   int useInterpolatedSize = Extend2dMeshIn3dVolumes();
 
-  HXT_INFO("Computing %smesh sizes...", useInterpolatedSize ? "interpolated " : "");
+  HXT_INFO("Computing %smesh sizes...",
+           useInterpolatedSize ? "interpolated " : "");
 
-#pragma omp parallel for schedule(dynamic)
+  int nthreads = getNumThreads();
+  bool exceptions = false;
+#pragma omp parallel for schedule(dynamic) num_threads(nthreads)
   for(size_t i = 0; i < numPts; i++) {
-    if(volume[i] < 0 || volume[i] >= allGR->size()) {
+    if(exceptions) continue;
+    if(volume[i] >= allGR->size()) {
       Msg::Error("Invalid volume tag %d in mesh size calculation", volume[i]);
       continue;
     }
     GRegion *gr = (*allGR)[volume[i]];
-    double lc = BGM_MeshSizeWithoutScaling(gr, 0, 0, pts[4 * i + 0],
-                                           pts[4 * i + 1], pts[4 * i + 2]);
-    if(useInterpolatedSize && pts[4 * i + 3] > 0.0)
-      pts[4 * i + 3] = std::min(pts[4 * i + 3], std::min(lcGlob, lc));
-    else
-      pts[4 * i + 3] = std::min(lcGlob, lc);
+    try { // OpenMP forbids leaving block via exception
+      double lc = BGM_MeshSizeWithoutScaling(gr, 0, 0, pts[4 * i + 0],
+                                             pts[4 * i + 1], pts[4 * i + 2]);
+      if(useInterpolatedSize && pts[4 * i + 3] > 0.0)
+        pts[4 * i + 3] = std::min(pts[4 * i + 3], std::min(lcGlob, lc));
+      else
+        pts[4 * i + 3] = std::min(lcGlob, lc);
+    } catch(...) {
+      exceptions = true;
+    }
   }
 
-  HXT_INFO("Done computing %smesh sizes", useInterpolatedSize ? "interpolated " : "");
+  if(exceptions) throw std::runtime_error(Msg::GetLastError());
+
+  HXT_INFO("Done computing %smesh sizes",
+           useInterpolatedSize ? "interpolated " : "");
 
   return HXT_STATUS_OK;
 }
@@ -85,7 +107,19 @@ static HXTStatus getAllSurfaces(std::vector<GRegion *> &regions, HXTMesh *m,
     allSurfacesSet.insert(f.begin(), f.end());
     allSurfacesSet.insert(f_e.begin(), f_e.end());
   }
-  allSurfaces.insert(allSurfaces.begin(), allSurfacesSet.begin(),
+
+  // verify that all elements are triangles
+  for(auto const &gf : allSurfacesSet) {
+    if(gf->quadrangles.size() != 0 || gf->polygons.size() != 0) {
+      std::size_t num = gf->quadrangles.size() + gf->polygons.size();
+      Msg::Error("Surface %d contains %zu elements which are not triangles. "
+                 "The HXT 3D meshing algorithm only supports triangles.",
+                 gf->tag(), num);
+      return HXT_STATUS_ERROR;
+    }
+  }
+
+  allSurfaces.insert(allSurfaces.end(), allSurfacesSet.begin(),
                      allSurfacesSet.end());
 
   if(!m) return HXT_STATUS_OK;
@@ -97,9 +131,9 @@ static HXTStatus getAllSurfaces(std::vector<GRegion *> &regions, HXTMesh *m,
   for(std::size_t i = 0; i < regions.size(); i++) {
     std::vector<GFace *> const &f = regions[i]->faces();
     std::vector<GFace *> const &f_e = regions[i]->embeddedFaces();
-    for(size_t j = 0; j < f.size(); j++)
+    for(std::size_t j = 0; j < f.size(); j++)
       m->brep.surfacesPerVolume[counter++] = f[j]->tag();
-    for(size_t j = 0; j < f_e.size(); j++)
+    for(std::size_t j = 0; j < f_e.size(); j++)
       m->brep.surfacesPerVolume[counter++] = f_e[j]->tag();
   }
 
@@ -133,7 +167,7 @@ static HXTStatus getAllCurves(std::vector<GRegion *> &regions,
     std::vector<GEdge *> const &r_e = regions[i]->embeddedEdges();
     allCurvesSet.insert(r_e.begin(), r_e.end());
   }
-  allCurves.insert(allCurves.begin(), allCurvesSet.begin(), allCurvesSet.end());
+  allCurves.insert(allCurves.end(), allCurvesSet.begin(), allCurvesSet.end());
 
   if(!m) return HXT_STATUS_OK;
 
@@ -144,9 +178,9 @@ static HXTStatus getAllCurves(std::vector<GRegion *> &regions,
   for(std::size_t i = 0; i < surfaces.size(); i++) {
     std::vector<GEdge *> const &f = surfaces[i]->edges();
     std::vector<GEdge *> const &f_e = surfaces[i]->embeddedEdges();
-    for(size_t j = 0; j < f.size(); j++)
+    for(std::size_t j = 0; j < f.size(); j++)
       m->brep.curvesPerSurface[counter++] = f[j]->tag();
-    for(size_t j = 0; j < f_e.size(); j++)
+    for(std::size_t j = 0; j < f_e.size(); j++)
       m->brep.curvesPerSurface[counter++] = f_e[j]->tag();
   }
 
@@ -159,10 +193,10 @@ static HXTStatus Hxt2Gmsh(std::vector<GRegion *> &regions, HXTMesh *m,
 {
   Msg::Debug("Start Hxt2Gmsh");
 
-  HXT_CHECK( hxtAlignedFree(&m->tetrahedra.neigh) );
-  HXT_CHECK( hxtAlignedFree(&m->tetrahedra.flag) );
-  HXT_CHECK( hxtAlignedFree(&m->points.node) );
-  HXT_CHECK( hxtAlignedFree(&m->points.color) );
+  HXT_CHECK(hxtAlignedFree(&m->tetrahedra.neigh));
+  HXT_CHECK(hxtAlignedFree(&m->tetrahedra.flag));
+  HXT_CHECK(hxtAlignedFree(&m->points.node));
+  HXT_CHECK(hxtAlignedFree(&m->points.color));
 
   std::map<uint32_t, GEdge *> i2e;
   std::map<uint32_t, GFace *> i2f;
@@ -172,17 +206,18 @@ static HXTStatus Hxt2Gmsh(std::vector<GRegion *> &regions, HXTMesh *m,
     HXT_CHECK(getAllSurfaces(regions, nullptr, allSurfaces));
     HXT_CHECK(getAllCurves(regions, allSurfaces, nullptr, allCurves));
 
-    for(size_t j = 0; j < allCurves.size(); j++) {
+    for(std::size_t j = 0; j < allCurves.size(); j++) {
       i2e[allCurves[j]->tag()] = allCurves[j];
       GEdge *ge = allCurves[j];
-      for(size_t i = 0; i < ge->lines.size(); i++) { delete ge->lines[i]; }
+      for(std::size_t i = 0; i < ge->lines.size(); i++) delete ge->lines[i];
       ge->lines.clear();
     }
 
-    for(size_t j = 0; j < allSurfaces.size(); j++) {
+    for(std::size_t j = 0; j < allSurfaces.size(); j++) {
       i2f[allSurfaces[j]->tag()] = allSurfaces[j];
       GFace *gf = allSurfaces[j];
-      for(size_t i = 0; i < gf->triangles.size(); i++) { delete gf->triangles[i]; }
+      for(std::size_t i = 0; i < gf->triangles.size(); i++)
+        delete gf->triangles[i];
       gf->triangles.clear();
     }
   }
@@ -190,7 +225,7 @@ static HXTStatus Hxt2Gmsh(std::vector<GRegion *> &regions, HXTMesh *m,
   c2v.resize(m->vertices.num, nullptr);
 
   uint32_t warning = 0;
-  for(size_t i = 0; i < m->lines.num; i++) {
+  for(std::size_t i = 0; i < m->lines.num; i++) {
     uint32_t c = m->lines.color[i];
     auto ge = i2e.find(c);
     if(ge == i2e.end()) {
@@ -215,10 +250,10 @@ static HXTStatus Hxt2Gmsh(std::vector<GRegion *> &regions, HXTMesh *m,
     }
     ge->second->lines.push_back(new MLine(c2v[i0], c2v[i1]));
   }
-  HXT_CHECK( hxtAlignedFree(&m->lines.node) );
-  HXT_CHECK( hxtAlignedFree(&m->lines.color) );
+  HXT_CHECK(hxtAlignedFree(&m->lines.node));
+  HXT_CHECK(hxtAlignedFree(&m->lines.color));
 
-  for(size_t i = 0; i < m->triangles.num; i++) {
+  for(std::size_t i = 0; i < m->triangles.num; i++) {
     uint32_t c = m->triangles.color[i];
     auto gf = i2f.find(c);
     if(gf == i2f.end()) {
@@ -249,11 +284,12 @@ static HXTStatus Hxt2Gmsh(std::vector<GRegion *> &regions, HXTMesh *m,
     }
     gf->second->triangles.push_back(new MTriangle(c2v[i0], c2v[i1], c2v[i2]));
   }
-  HXT_CHECK( hxtAlignedFree(&m->triangles.node) );
-  HXT_CHECK( hxtAlignedFree(&m->triangles.color) );
+  HXT_CHECK(hxtAlignedFree(&m->triangles.node));
+  HXT_CHECK(hxtAlignedFree(&m->triangles.color));
 
-#if defined(_OPENMP)
-  int nthreads = omp_get_max_threads();
+#if 0 // this seems to be buggy (cf. e.g. #2575)
+// #if defined(_OPENMP)
+  int nthreads = getNumThreads();
   if(nthreads > 1) {
     const uint32_t nR = regions.size();
     const uint32_t nV = m->vertices.num;
@@ -265,9 +301,9 @@ static HXTStatus Hxt2Gmsh(std::vector<GRegion *> &regions, HXTMesh *m,
                          sizeof(uint32_t)) );
     uint32_t* vR_all = hp_all + nR * (nthreads + 1); // color per point and per thread
 
-    #pragma omp parallel
+#pragma omp parallel num_threads(nthreads)
     {
-      #pragma omp single
+#pragma omp single
       {
         nthreads = omp_get_num_threads(); /* the real number of threads */
         ht_tot = ht_all + nR * nthreads;
@@ -280,7 +316,7 @@ static HXTStatus Hxt2Gmsh(std::vector<GRegion *> &regions, HXTMesh *m,
       uint32_t* vR_this = vR_all + nV * threadID;
 
       // count the number of tetrahedra in each region in parallel
-      #pragma omp for schedule(static)
+#pragma omp for schedule(static)
       for(size_t i = 0; i < m->tetrahedra.num; i++) {
         uint32_t c = m->tetrahedra.color[i];
         if(c >= nR) continue;
@@ -294,7 +330,7 @@ static HXTStatus Hxt2Gmsh(std::vector<GRegion *> &regions, HXTMesh *m,
         }
       }
 
-      #pragma omp for schedule(static)
+#pragma omp for schedule(static)
       for(uint32_t pt = 0; pt < nV; pt++) {
         if(c2v[pt]) continue;
 
@@ -306,15 +342,15 @@ static HXTStatus Hxt2Gmsh(std::vector<GRegion *> &regions, HXTMesh *m,
           }
         }
         color--;
-  #ifdef DEBUG
+#ifdef DEBUG
         if(color >= nR)
           exit(HXT_ERROR_MSG(HXT_STATUS_ERROR, "no volume or color for pt %u", pt));
-  #endif
+#endif
         vR_all[pt] = color;
         hp_this[color]++;
       }
 
-      #pragma omp for
+#pragma omp for
       for(uint32_t c2 = 0; c2 < 2 * nR; c2++) { // parallelism x 2 :p
         uint32_t c = c2 >> 1;
         if(c2 & 1) {
@@ -337,7 +373,7 @@ static HXTStatus Hxt2Gmsh(std::vector<GRegion *> &regions, HXTMesh *m,
         }
       }
 
-      #pragma omp for schedule(static)
+#pragma omp for schedule(static)
       for(uint32_t pt = 0; pt < nV; pt++) {
         if(c2v[pt]) continue;
 
@@ -348,8 +384,7 @@ static HXTStatus Hxt2Gmsh(std::vector<GRegion *> &regions, HXTMesh *m,
         gr->mesh_vertices[hp_this[c]++] = c2v[pt];
       }
 
-
-      #pragma omp for schedule(static)
+#pragma omp for schedule(static)
       for(size_t i = 0; i < m->tetrahedra.num; i++) {
         uint32_t c = m->tetrahedra.color[i];
         if(c >= nR) continue;
@@ -366,20 +401,29 @@ static HXTStatus Hxt2Gmsh(std::vector<GRegion *> &regions, HXTMesh *m,
   else
 #endif
   {
-    for(size_t i = 0; i < m->tetrahedra.num; i++) {
+    std::vector<std::size_t> numtet(regions.size(), 0);
+    for(std::size_t i = 0; i < m->tetrahedra.num; i++) {
       uint16_t c = m->tetrahedra.color[i];
-      if(c >= regions.size())
-        continue;
+      if(c >= numtet.size()) continue;
+      numtet[c]++;
+    }
+    for(std::size_t i = 0; i < numtet.size(); i++) {
+      regions[i]->tetrahedra.reserve(numtet[i]);
+      regions[i]->mesh_vertices.reserve(numtet[i] / 6);
+    }
+
+    for(std::size_t i = 0; i < m->tetrahedra.num; i++) {
+      uint16_t c = m->tetrahedra.color[i];
+      if(c >= regions.size()) continue;
 
       GRegion *gr = regions[c];
       MVertex *vv[4];
       uint32_t *nodes = &m->tetrahedra.node[4 * i];
       for(int j = 0; j < 4; j++) {
-        if(c2v[nodes[j]]){
+        if(c2v[nodes[j]]) {
           vv[j] = c2v[nodes[j]];
           continue;
         }
-
         double *x = &m->vertices.coord[4 * nodes[j]];
         vv[j] = new MVertex(x[0], x[1], x[2], gr);
         gr->mesh_vertices.push_back(vv[j]);
@@ -398,7 +442,7 @@ HXTStatus Gmsh2Hxt(std::vector<GRegion *> &regions, HXTMesh *m,
                    std::map<MVertex *, uint32_t> &v2c,
                    std::vector<MVertex *> &c2v)
 {
-  std::set<MVertex *> all;
+  std::set<MVertex *, MVertexPtrLessThan> all;
   std::vector<GFace *> surfaces;
   std::vector<GEdge *> curves;
   std::vector<GVertex *> points;
@@ -415,7 +459,7 @@ HXTStatus Gmsh2Hxt(std::vector<GRegion *> &regions, HXTMesh *m,
     for(GVertex *gv : gr->embeddedVertices()) {
       points.push_back(gv);
       npts += gv->points.size();
-      for(size_t i = 0; i < gv->points.size(); i++) {
+      for(std::size_t i = 0; i < gv->points.size(); i++) {
         MVertex *v = gv->points[i]->getVertex(0);
         all.insert(v);
         if(gv->prescribedMeshSizeAtVertex() != MAX_LC)
@@ -424,19 +468,19 @@ HXTStatus Gmsh2Hxt(std::vector<GRegion *> &regions, HXTMesh *m,
     }
   }
 
-  for(size_t j = 0; j < curves.size(); j++) {
+  for(std::size_t j = 0; j < curves.size(); j++) {
     GEdge *ge = curves[j];
     nedg += ge->lines.size();
-    for(size_t i = 0; i < ge->lines.size(); i++) {
+    for(std::size_t i = 0; i < ge->lines.size(); i++) {
       all.insert(ge->lines[i]->getVertex(0));
       all.insert(ge->lines[i]->getVertex(1));
     }
   }
 
-  for(size_t j = 0; j < surfaces.size(); j++) {
+  for(std::size_t j = 0; j < surfaces.size(); j++) {
     GFace *gf = surfaces[j];
     ntri += gf->triangles.size();
-    for(size_t i = 0; i < gf->triangles.size(); i++) {
+    for(std::size_t i = 0; i < gf->triangles.size(); i++) {
       all.insert(gf->triangles[i]->getVertex(0));
       all.insert(gf->triangles[i]->getVertex(1));
       all.insert(gf->triangles[i]->getVertex(2));
@@ -447,7 +491,7 @@ HXTStatus Gmsh2Hxt(std::vector<GRegion *> &regions, HXTMesh *m,
   HXT_CHECK(
     hxtAlignedMalloc(&m->vertices.coord, 4 * m->vertices.num * sizeof(double)));
 
-  size_t count = 0;
+  std::size_t count = 0;
   c2v.resize(all.size());
   for(MVertex *v : all) {
     m->vertices.coord[4 * count + 0] = v->x();
@@ -470,9 +514,9 @@ HXTStatus Gmsh2Hxt(std::vector<GRegion *> &regions, HXTMesh *m,
   HXT_CHECK(
     hxtAlignedMalloc(&m->points.color, (m->points.num) * sizeof(uint32_t)));
   index = 0;
-  for(size_t j = 0; j < points.size(); j++) {
+  for(std::size_t j = 0; j < points.size(); j++) {
     GVertex *gv = points[j];
-    for(size_t i = 0; i < gv->points.size(); i++) {
+    for(std::size_t i = 0; i < gv->points.size(); i++) {
       m->points.node[index] = v2c[gv->points[i]->getVertex(0)];
       m->points.color[index] = gv->tag();
       index++;
@@ -485,9 +529,9 @@ HXTStatus Gmsh2Hxt(std::vector<GRegion *> &regions, HXTMesh *m,
   HXT_CHECK(
     hxtAlignedMalloc(&m->lines.color, (m->lines.num) * sizeof(uint32_t)));
   index = 0;
-  for(size_t j = 0; j < curves.size(); j++) {
+  for(std::size_t j = 0; j < curves.size(); j++) {
     GEdge *ge = curves[j];
-    for(size_t i = 0; i < ge->lines.size(); i++) {
+    for(std::size_t i = 0; i < ge->lines.size(); i++) {
       m->lines.node[2 * index + 0] = v2c[ge->lines[i]->getVertex(0)];
       m->lines.node[2 * index + 1] = v2c[ge->lines[i]->getVertex(1)];
       m->lines.color[index] = ge->tag();
@@ -501,9 +545,9 @@ HXTStatus Gmsh2Hxt(std::vector<GRegion *> &regions, HXTMesh *m,
   HXT_CHECK(hxtAlignedMalloc(&m->triangles.color,
                              (m->triangles.num) * sizeof(uint32_t)));
   index = 0;
-  for(size_t j = 0; j < surfaces.size(); j++) {
+  for(std::size_t j = 0; j < surfaces.size(); j++) {
     GFace *gf = surfaces[j];
-    for(size_t i = 0; i < gf->triangles.size(); i++) {
+    for(std::size_t i = 0; i < gf->triangles.size(); i++) {
       m->triangles.node[3 * index + 0] = v2c[gf->triangles[i]->getVertex(0)];
       m->triangles.node[3 * index + 1] = v2c[gf->triangles[i]->getVertex(1)];
       m->triangles.node[3 * index + 2] = v2c[gf->triangles[i]->getVertex(2)];
@@ -525,16 +569,18 @@ static HXTStatus _meshGRegionHxt(std::vector<GRegion *> &regions)
   std::vector<MVertex *> c2v;
   Gmsh2Hxt(regions, mesh, v2c, c2v);
 
+  int nthreads = getNumThreads();
+
   HXTTetMeshOptions options = {
-    0, // int defaultThreads;
-    0, // int delaunayThreads;
-    0, // int improveThreads;
-    1, // int reproducible;
+    nthreads, // int defaultThreads;
+    nthreads, // int delaunayThreads;
+    nthreads, // int improveThreads;
+    CTX::instance()->mesh.reproducible, // int reproducible;
     (Msg::GetVerbosity() > 5) ? 2 : 1, // int verbosity;
     1, // int stat;
     1, // int refine;
     CTX::instance()->mesh.optimize, // int optimize;
-    CTX::instance()->mesh.toleranceInitialDelaunay,// tolerance for tetgen
+    CTX::instance()->mesh.toleranceInitialDelaunay, // tolerance for tetgen
     {
       // quality
       nullptr, // double (*callback)(.., userData)
@@ -566,7 +612,8 @@ int meshGRegionHxt(std::vector<GRegion *> &regions)
 }
 
 static HXTStatus _delaunayMeshIn3DHxt(std::vector<MVertex *> &verts,
-                                      std::vector<MTetrahedron *> &tets)
+                                      std::vector<MTetrahedron *> &tets,
+                                      const std::vector<MTriangle> &triangles)
 {
   HXTMesh *mesh;
   HXT_CHECK(hxtMeshCreate(&mesh));
@@ -583,6 +630,8 @@ static HXTStatus _delaunayMeshIn3DHxt(std::vector<MVertex *> &verts,
   mesh->vertices.num = nvert;
   mesh->vertices.size = nvert;
 
+  int nthreads = getNumThreads();
+
   HXTDelaunayOptions delOptions = {
     nullptr, // bbox
     nullptr, // nodalSizes
@@ -590,9 +639,10 @@ static HXTStatus _delaunayMeshIn3DHxt(std::vector<MVertex *> &verts,
     0, // insertionFirst
     0, // partitionability
     1, // perfectDelaunay
+    0, // allowOuterInsertion
     0, // verbosity
     0, // reproducible
-    0 // delaunayThreads (0 = omp_get_max_threads)
+    nthreads // delaunayThreads (0 = omp_get_max_threads)
   };
 
   // HXT_CHECK(hxtDelaunay(mesh, &delOptions));
@@ -607,6 +657,38 @@ static HXTStatus _delaunayMeshIn3DHxt(std::vector<MVertex *> &verts,
     hxtDelaunaySteadyVertices(mesh, &delOptions, nodeInfo, mesh->vertices.num));
   HXT_CHECK(hxtAlignedFree(&nodeInfo));
 
+  size_t nTriangles = triangles.size();
+
+  if(nTriangles > 0) {
+    HXT_CHECK(hxtAlignedMalloc(&mesh->triangles.node,
+                               3 * nTriangles * sizeof(uint32_t)));
+    HXT_CHECK(hxtAlignedMalloc(&mesh->triangles.color,
+                               nTriangles * sizeof(uint32_t)));
+    for(size_t i = 0; i < nTriangles; i++) {
+      mesh->triangles.node[3 * i + 0] = triangles[i].getVertex(0)->getIndex();
+      mesh->triangles.node[3 * i + 1] = triangles[i].getVertex(1)->getIndex();
+      mesh->triangles.node[3 * i + 2] = triangles[i].getVertex(2)->getIndex();
+      mesh->triangles.color[i] = 0;
+    }
+    mesh->triangles.num = nTriangles;
+    mesh->triangles.size = nTriangles;
+
+    hxt_boundary_recovery(mesh, 0.0);
+
+    if(nvert < mesh->vertices.num) {
+      // Steiner points!
+      std::size_t j = nvert;
+      for(size_t i = nvert; i < mesh->vertices.num; ++i) {
+        MVertex *sterinerPoint = new MVertex(mesh->vertices.coord[4 * i + 0],
+                                             mesh->vertices.coord[4 * i + 1],
+                                             mesh->vertices.coord[4 * i + 2]);
+        sterinerPoint->setIndex(j);
+        verts.push_back(sterinerPoint);
+        j++;
+      }
+    }
+  }
+
   for(size_t i = 0; i < mesh->tetrahedra.num; i++) {
     if(mesh->tetrahedra.node[i * 4 + 3] != UINT32_MAX) {
       uint32_t myColor = mesh->tetrahedra.color ? mesh->tetrahedra.color[i] : 0;
@@ -623,11 +705,12 @@ static HXTStatus _delaunayMeshIn3DHxt(std::vector<MVertex *> &verts,
 }
 
 void delaunayMeshIn3DHxt(std::vector<MVertex *> &v,
-                         std::vector<MTetrahedron *> &tets)
+                         std::vector<MTetrahedron *> &tets,
+                         const std::vector<MTriangle> &triangles)
 {
   Msg::Info("Tetrahedrizing %d nodes...", v.size());
   double t1 = Cpu(), w1 = TimeOfDay();
-  _delaunayMeshIn3DHxt(v, tets);
+  _delaunayMeshIn3DHxt(v, tets, triangles);
   double t2 = Cpu(), w2 = TimeOfDay();
   Msg::Info("Done tetrahedrizing %d nodes (Wall %gs, CPU %gs)", v.size(),
             w2 - w1, t2 - t1);
@@ -642,7 +725,8 @@ int meshGRegionHxt(std::vector<GRegion *> &regions)
 }
 
 void delaunayMeshIn3DHxt(std::vector<MVertex *> &v,
-                         std::vector<MTetrahedron *> &tets)
+                         std::vector<MTetrahedron *> &tets,
+                         const std::vector<MTriangle> &triangles)
 {
   Msg::Error("Gmsh should be compiled with Hxt to enable this option");
 }
