@@ -1,14 +1,16 @@
-// Gmsh - Copyright (C) 1997-2022 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2025 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file in the Gmsh root directory for license information.
 // Please report all issues on https://gitlab.onelab.info/gmsh/gmsh/issues.
 //
 // Contributed by Ismail Badia.
-// Reference :  "Higher-Order Finite Element  Methods"; Pavel Solin, Karel
-// Segeth ,
-//                 Ivo Dolezel , Chapman and Hall/CRC; Edition : Har/Cdr (2003).
 
+// Reference :  "Higher-Order Finite Element  Methods"; Pavel Solin, Karel
+// Segeth, Ivo Dolezel, Chapman and Hall/CRC; Edition : Har/Cdr (2003).
+
+#include <stdexcept>
 #include "HierarchicalBasisH1Tetra.h"
+
 HierarchicalBasisH1Tetra::HierarchicalBasisH1Tetra(int order)
 {
   _nvertex = 4;
@@ -42,7 +44,7 @@ double HierarchicalBasisH1Tetra::_affineCoordinate(const int &j,
   case(2): return -0.5 * (1 + u + v + w);
   case(3): return 0.5 * (1 + u);
   case(4): return 0.5 * (1 + w);
-  default: throw std::string("j must be : 1<=j<=4");
+  default: throw std::runtime_error("j must be : 1<=j<=4");
   }
 }
 
